#
# MicroTESK for PowerPC
#
# Copyright (c) 2016-2018 Institute for System Programming of the Russian Academy of Sciences
# All Rights Reserved
#
# Institute for System Programming of the Russian Academy of Sciences (ISP RAS)
# 25 Alexander Solzhenitsyn st., Moscow, 109004, Russia
# http://www.ispras.ru
#

require_relative 'powerpc_base'

#
# Description:
#
# This small tests for ALU instructions.
#

class InstructionALU < PowerPCBaseTemplate

  def run
    trace "Run ALU instruction:"

    trace "\nPowerPC Arithmetic:\n"

    addi r1, r0, 0x11
    trace "(add): r1 = %x", gpr_observer(1)
    add r2, r1, r1
    trace "(add): r2 = %x", gpr_observer(2)
    addi r3, r0, 0x1
    trace "(add): r3 = %x", gpr_observer(3)

    ori r4, r2, 0x11
    trace "(ori): r4 = %x", gpr_observer(4)
    add_d r5, r4, r4
    trace "(add.): r5 = %x", gpr_observer(5)
    addo r2, r2, r2
    trace "(addo): r2 = %x", gpr_observer(2)
    addo_d r2, r2, r1
    trace "(addo.): r2 = %x", gpr_observer(2)

    subf r6, r1, r5
    trace "(sub): r6 = %x", gpr_observer(6)
    subf_d r7, r5, r1
    trace "(sub.): r7 = %x", gpr_observer(7)
    subf r8, r7, r4
    trace "(sub): r8 = %x", gpr_observer(8)
    subfo r9, r6, r8
    trace "(subo): r9 = %x", gpr_observer(9)
    subfo_d r10, r6, r9
    trace "(subo.): r10 = %x", gpr_observer(10)
    
    neg r11, r10
    trace "(neg): r11 = %x", gpr_observer(11)
    neg_d r11, r11
    trace "(neg.): r11 = %x", gpr_observer(11)
    addi r11, r0, 0x0
    trace "(add): r11 = %x", gpr_observer(11)
    nego r11, r11
    trace "(nego): r11 = %x", gpr_observer(11)
    nego_d r12, r9
    trace "(nego.): r12 = %x", gpr_observer(12)
    
    xor r13, r12, r10
    trace "(xor): r13 = %x", gpr_observer(13)
    xor_d r13, r13, r11
    trace "(xor.): r13 = %x", gpr_observer(13)
    
    rlwinm r14, r13, 31, 16, 15
    trace "(rlwinm): r14 = %x", gpr_observer(14)
    srwi r14, r13, 1
    trace "(srwi): r14 = %x", gpr_observer(14)
    srwi r14, r13, 2
    trace "(srwi): r14 = %x", gpr_observer(14)
    srwi_d r14, r10, 16
    trace "(srwi.): r14 = %x", gpr_observer(14)
    slwi r14, r10, 16
    trace "(slwi): r14 = %x", gpr_observer(14)
    slwi_d r14, r13, 1
    trace "(slwi.): r14 = %x", gpr_observer(14)
    extlwi r14, r10, 10, 5 
    trace "(extlwi): r14 = %x", gpr_observer(14)
    extlwi_d r14, r10, 15, 25 
    trace "(extlwi.): r14 = %x", gpr_observer(14)
    extrwi r14, r10, 10, 5 
    trace "(extrwi): r14 = %x", gpr_observer(14)
    extrwi_d r14, r10, 15, 15 
    trace "(extrwi.): r14 = %x", gpr_observer(14)
    rotlwi r14, r10, 8
    trace "(rotlwi): r14 = %x", gpr_observer(14)
    rotlwi_d r14, r10, 28
    trace "(rotlwi.): r14 = %x", gpr_observer(14)
    rotrwi r14, r10, 8
    trace "(rotrwi): r14 = %x", gpr_observer(14)
    rotrwi_d r14, r10, 28
    trace "(rotrwi.): r14 = %x", gpr_observer(14)
    clrlwi r14, r10, 8
    trace "(clrlwi): r14 = %x", gpr_observer(14)
    clrlwi_d r14, r10, 28
    trace "(clrlwi.): r14 = %x", gpr_observer(14)
    clrrwi r14, r10, 8
    trace "(clrrwi): r14 = %x", gpr_observer(14)
    clrrwi_d r14, r10, 28
    trace "(clrrwi.): r14 = %x", gpr_observer(14)
    clrlslwi r14, r10, 16, 8 
    trace "(clrlslwi): r14 = %x", gpr_observer(14)
    clrlslwi_d r14, r10, 24, 12 
    trace "(clrlslwi.): r14 = %x", gpr_observer(14)
    
    OR r15, r13, r8
    trace "(or): r15 = %x", gpr_observer(15)
    or_d r15, r13, r8
    trace "(or.): r15 = %x", gpr_observer(15)
    mr r15, r12
    trace "(mr): r15 = %x", gpr_observer(15)
    mr_d r15, r14
    trace "(mr.): r15 = %x", gpr_observer(15)
    srawi r15, r12, 16
    trace "(srawi): r15 = %x", gpr_observer(15)
    srawi_d r15, r13, 1
    trace "(srawi.): r15 = %x", gpr_observer(15)
    
    trace "cr = %s", cr_observer()
    cmpw cr0, r15, r14
    trace "(cmpw): cr = %s", cr_observer()
    cmpw cr3, r15, r15
    trace "(cmpw): cr = %s", cr_observer()
    cmpw cr7, r15, r10
    trace "(cmpw): cr = %s", cr_observer()
    addi r1, r0, 0x5BCD
    addi r2, r1, 1
    cmpwi cr0, r1, 0x5BCD
    trace "(cmpwi): cr = %s", cr_observer()
    cmpwi cr1, r1, -100
    trace "(cmpwi): cr = %s", cr_observer()
    cmpwi cr2, r15, 0x6BCD
    trace "(cmpwi): cr = %s", cr_observer()
    
    cmplw cr0, r15, r14
    trace "(cmplw): cr = %s", cr_observer()
    cmplw cr3, r15, r15
    trace "(cmplw): cr = %s", cr_observer()
    cmplw cr7, r15, r10
    trace "(cmplw): cr = %s", cr_observer()
    cmplwi cr0, r1, 0x5BCD
    trace "(cmplwi): cr = %s", cr_observer()
    cmplwi cr1, r1, -100
    trace "(cmplwi): cr = %s", cr_observer()
    cmplwi cr2, r15, 0x6BCD
    trace "(cmplwi): cr = %s", cr_observer()
    
    mulhw r17,r14,r14
    trace "(mulhw): r17 = %x", gpr_observer(17)
    mulhw_d r17,r14,r14
    trace "(mulhw.): cr = %s", cr_observer()
    mulhwu r17,r14,r14
    trace "(mulhw): r17 = %x", gpr_observer(17)
    mulhwu_d r17,r14,r14
    trace "(mulhw.): cr = %s", cr_observer()
    mullw r17,r14,r14
    trace "(mullw): r17 = %x", gpr_observer(17)
    mullw_d r17,r14,r14
    trace "(mullw.): cr = %s", cr_observer()
    mullwo r17,r14,r14
    trace "(mullwo): r17 = %x", gpr_observer(17)
    mullwo_d r17,r14,r14
    trace "(mullwo.): cr = %s", cr_observer()
    mulli r17,r14,2
    trace "(mulli): r17 = %x", gpr_observer(17)
    mulli r17,r14,-1
    trace "(mulli): r17 = %x", gpr_observer(17)
    
    addic r18, r0, 0x11
    trace "(addic): r18 = %x", gpr_observer(18)
    addic_d r18, r18, 0x11
    trace "(addic.): r18 = %x", gpr_observer(18)
    subic r18, r18, 1
    trace "(subic): r18 = %x", gpr_observer(18)
    subic_d r18, r18, 1
    trace "(subic.): r18 = %x", gpr_observer(18)
    subi r18, r18, 1
    trace "(subi): r18 = %x", gpr_observer(18)
    addc r19, r18, r18
    trace "(addc): r19 = %x", gpr_observer(19)
    addc_d r19, r19, r18
    trace "(addc.): r19 = %x", gpr_observer(19)
    addco r19, r19, r19
    trace "(addco): r19 = %x", gpr_observer(19)
    addco_d r19, r18, r19
    trace "(addco.): r19 = %x", gpr_observer(19)
    li r19, 0xBA 
    trace "(li): r19 = %x", gpr_observer(19)
    lis r19, 0xDC 
    trace "(lis): r19 = %x", gpr_observer(19)
    addis r19, r19, 1
    trace "(addis): r19 = %x", gpr_observer(19)
    subis r19, r19, 2
    trace "(subis): r19 = %x", gpr_observer(19)
    lis r20, 0xFFFF
    addc r20, r20, r19
    trace "r20 = %x", gpr_observer(20)
    adde r20, r20, r20
    trace "(adde): r20 = %x", gpr_observer(20)
    addme r20, r20
    trace "(addme): r20 = %x", gpr_observer(20)
    addc r20, r20, r17
    trace "r20 = %x", gpr_observer(20)
    addze r20, r20
    trace "(addze): r20 = %x", gpr_observer(20)
    lis r20, 0xFFFF
    addc r20, r20, r19
    trace "r20 = %x", gpr_observer(20)
    adde_d r20, r20, r20
    trace "(adde.): r20 = %x", gpr_observer(20)
    addme_d r20, r20
    trace "(addme.): r20 = %x", gpr_observer(20)
    addc r20, r20, r17
    trace "r20 = %x", gpr_observer(20)
    addze_d r20, r20
    trace "(addze.): r20 = %x", gpr_observer(20)
    lis r20, 0xFFFF
    addc r20, r20, r19
    trace "r20 = %x", gpr_observer(20)
    addeo r20, r20, r20
    trace "(addeo): r20 = %x", gpr_observer(20)
    addmeo r20, r20
    trace "(addmeo): r20 = %x", gpr_observer(20)
    addc r20, r20, r17
    trace "r20 = %x", gpr_observer(20)
    addzeo r20, r20
    trace "(addzeo): r20 = %x", gpr_observer(20)
    lis r20, 0xFFFF
    addc r20, r20, r19
    trace "r20 = %x", gpr_observer(20)
    addeo_d r20, r20, r20
    trace "(addeo.): r20 = %x", gpr_observer(20)
    addmeo_d r20, r20
    trace "(addmeo.): r20 = %x", gpr_observer(20)
    addc r20, r20, r17
    trace "r20 = %x", gpr_observer(20)
    addzeo_d r20, r20
    trace "(addzeo.): r20 = %x", gpr_observer(20)
    subfc r20, r20, r19
    trace "(subfc): r20 = %x", gpr_observer(20)
    subfc_d r20, r20, r19
    trace "(subfc.): r20 = %x", gpr_observer(20)
    subfco r20, r20, r19
    trace "(subfco): r20 = %x", gpr_observer(20)
    subfco_d r20, r20, r19
    trace "(subfco.): r20 = %x", gpr_observer(20)
    subfic r19,r19,1
    trace "(subfic): r19 = %x", gpr_observer(19)
    subfe r20, r20, r19
    trace "(subfe): r20 = %x", gpr_observer(20)
    subfe_d r20, r20, r19
    trace "(subfe.): r20 = %x", gpr_observer(20)
    subfeo r20, r20, r19
    trace "(subfeo): r20 = %x", gpr_observer(20)
    subfeo_d r20, r20, r19
    trace "(subfeo.): r20 = %x", gpr_observer(20)
    subfme r20, r20
    trace "(subfme): r20 = %x", gpr_observer(20)
    subfme_d r20, r20
    trace "(subfme.): r20 = %x", gpr_observer(20)
    subfmeo r20, r20
    trace "(subfmeo): r20 = %x", gpr_observer(20)
    subfmeo_d r20, r20
    trace "(subfmeo.): r20 = %x", gpr_observer(20)
    subfze r20, r20
    trace "(subfze): r20 = %x", gpr_observer(20)
    subfze_d r20, r20
    trace "(subfze.): r20 = %x", gpr_observer(20)
    subfzeo r20, r20
    trace "(subfzeo): r20 = %x", gpr_observer(20)
    subfzeo_d r20, r20
    trace "(subfzeo.): r20 = %x", gpr_observer(20)
    li r20, 16
    divw r21, r19, r20
    trace "(divw): r21 = %x", gpr_observer(21)
    divw_d r21, r21, r20
    trace "(divw.): r21 = %x", gpr_observer(21)
    li r20, 0
    divwo r21, r19, r20
    trace "(divwo): r21 = %x", gpr_observer(21)
    li r20, -1
    divwo_d r21, r19, r20
    trace "(divwo.): r21 = %x", gpr_observer(21)
    divwu r21, r21, r20
    trace "(divwu): r21 = %x", gpr_observer(21)
    divwu_d r21,r20,r21
    trace "(divwu.): r21 = %x", gpr_observer(21)
    divwuo r21,r21,r19
    trace "(divwuo): r21 = %x", gpr_observer(21)
    divwuo r21,r21,r19
    trace "(divwuo.): r21 = %x", gpr_observer(21)
    li r21, 5
    li r22, 6
    #and r22,r21,r22
    #trace "(and): r22 = %x", r22
    and_d r23,r21,r22
    trace "(and.): r23 = %x", r23
    oris r24,r21,7
    trace "(oris): r24 = %x", r24
    andi_d r23, r23, 15
    trace "(andi.): r23 = %x", r23
    andis_d r24, r24, 6
    trace "(andis.): r24 = %x", r24
    andc r24,r21,r22
    trace "(andc): r24 = %x", r24
    andc_d r24,r21,r23
    trace "(andc.): r24 = %x", r24
    orc r24,r21,r22
    trace "(orc): r24 = %x", r24
    orc_d r24,r21,r23
    trace "(orc.): r24 = %x", r24
    xori r24,r21,15
    trace "(xori): r24 = %x", r24
    xoris r24,r21,15
    trace "(xoris): r24 = %x", r24
    nand r24,r21,r22
    trace "(nand): r24 = %x", r24
    nand_d r24,r21,r23
    trace "(nand.): r24 = %x", r24
    eqv r24,r21,r22
    trace "(eqv): r24 = %x", r24
    eqv_d r24,r21,r23
    trace "(eqv.): r24 = %x", r24
    nor r24,r21,r22
    trace "(nor): r24 = %x", r24
    nor_d r24,r21,r23
    trace "(nor.): r24 = %x", r24
    #not r24,r24
    #trace "(not): r24 = %x", r24
    not_d r24,r24
    trace "(not.): r24 = %x", r24
    li r23, 150
    extsb r24, r23
    trace "(extsb): r24 = %x", r24
    extsb r24, r21
    trace "(extsb.): r24 = %x", r24
    li r23, -500
    extsh r24, r23
    trace "(extsh): r24 = %x", r24
    extsh r24, r21
    trace "(extsh.): r24 = %x", r24
    inslwi_d r24, r24, 1, 1  
    trace "(inslwi.): r24 = %x", r24
    insrwi r24, r21, 1, 0  
    trace "(insrwi): r24 = %x", r24
    rotlw r24, r24, r23
    trace "(rotlw): r24 = %x", r24
    rotlw_d r24, r22, r21
    trace "(rotlw.): r24 = %x", r24
    slw r24, r23, r21
    trace "(slw.): r24 = %x", r24
    slw_d r24, r24, r22
    trace "(slw.): r24 = %x", r24
    sraw r24, r24, r21
    trace "(sraw): r24 = %x", r24
    sraw_d r24, r24, r22
    trace "(sraw.): r24 = %x", r24
    srw r24, r24, r21
    trace "(srw): r24 = %x", r24
    srw_d r24, r24, r21
    trace "(srw.): r24 = %x", r24
    nop
    nop
  end

end
