# MicroTESK Installation Guide

## System Requirements

MicroTESK is a set of Java-based utilities that are run from the command line.
It can be used on Windows, Linux and OS X machines that have JDK 1.7 or later installed.
To build MicroTESK from source code or to build generated Java models,
Apache Ant version 1.8 or later is required.
To generate test data based on constraints, MicroTESK needs Z3 or CVC4 solvers.

## Installation Steps

1. Download from http://forge.ispras.ru/projects/microtesk/files and unpack the MicroTESK
   installation package (the `.tar.gz` file, latest release) to your computer.

2. Download and install constraint solver tools to the `<installation dir>/tools` directory
   (see the `tools/README.md` document for details).

3. Declare the `MICROTESK_HOME` environment variable and set its value to the path to the
   installation directory(see the Setting Environment Variables section).

4. Set the `<installation dir>/bin` as the working directory (add the path to the `PATH`
  environment variable) to be able to run MicroTESK utilities from any path.

5. Now you can run the `compile.sh` (or `.bat`) script to create a microprocessor model
   and the `generate.sh` (or `.bat`) script to generate tests for this model.

## Setting Environment Variables

### Windows

1. Open the `System Properties` window.
2. Switch to the `Advanced` tab.
3. Click on `Environment Variables`.
4. Click `New...` under `System Variables`.
5. In the `New System Variable` dialog specify variable name as `MICROTESK_HOME` and variable value
   as `<instrallation dir>`.
6. Click `OK` on all open windows.
7. Reopen the command prompt window.

### Linux and OS X

Add the command below to the `~.bash_profile` (Linux) or `~/.profile` (OS X) file:

```
$ export MICROTESK_HOME=<installation dir>
```

To start editing the file, type `vi ~/.bash_profile` (or `vi ~/.profile`).
Changes will be applied after restarting the command-line terminal or reboot.
You can also execute the command in your command-line terminal to make temporary changes.

## Installation Directory Structure

The MicroTESK installation directory contains the following subdirectories:



| Directory | Description |
| :-------- | :-----------|
| `arch` | Examples of microprocessor specifications and test templates for the described designs |
| `bin`  | Scripts to run features of MicroTESK (modeling and test generation)                    |
| `doc`  | Documentation on MicroTESK                                                             |
| `etc`  | MicroTESK configuration files                                                          |
| `gen`  | Generated Java models of the specified microprocessor designs                          |
| `lib`  | JAR files and Ruby scripts to perform modeling and test generation tasks               |
| `src`  | Source code of MicroTESK                                                               |

## Running MicroTESK

To generate a Java model of a microprocessor from its nML specification, a user needs to run the
`compile.sh` script (Unix, Linux, OS X) or the `compile.bat` script (Windows).
For example, the following command generates a model for the miniMIPS specification:

```
$ sh bin/compile.sh arch/minimips/model/minimips.nml arch/minimips/model/mmu/minimips.mmu
```

**NOTE:** Models for all demo specifications are already built and included in the MicroTESK
distribution package.
So a user can start working with MicroTESK from generating test programs for these models.

To generate a test program, a user needs to use the generate.sh script (Unix,
Linux, OS X) or the generate.bat script (Windows). The scripts require the
following mandatory parameters:

* model name;
* test template file path.

For example, the following command runs the `euclid.rb` test template for the miniMIPS model
generated by the command from the previous example and saves the generated test program to the
`test.asm` file:

```
$ sh bin/generate.sh minimips arch/minimips/templates/euclid.rb
```

Also, MicroTESK uses a set of additional parameters that can be used to customize its behavior.
To get more information about them, the user needs to run MicroTESK with the `--help` key.
